package test.portlet.consumer;

import com.vaadin.Application;
import com.vaadin.service.ApplicationContext.TransactionListener;

public class ConsumerController {
    private ConsumerApplication application;
    private static TransactionListener tl;

    private static ThreadLocal<ConsumerController> controller = new ThreadLocal<ConsumerController>();

    public ConsumerController(ConsumerApplication application) {
        this.application = application;
    }

    public void init() {
        // get the controller from the application and set it for the initial phase, e.g.
        // for drawing the window
        ConsumerController ctr = ((ConsumerApplication) application).getController();
        controller.set(ctr);
        // create and add a transaction listener to be able to assign a controller instance to each
        // request from the client. On each transaction the controller must be set again. Otherwise
        // a NullPointerException will be thrown.
        tl = new WindowTransactionListener();
        this.application.getContext().addTransactionListener(tl);
        if (application.isInPortletContext()) {
            // do some stuff here
        } else {
            // not within portlet container. Return some simulation stuff!
            // return SimulationFactory.getTaskDescriptionList();
        }
    }

    public static ConsumerController get() {
        return controller.get();
    }

    public Application getApplication() {
        return application;
    }

    public void removeTransactionListener() {
        ConsumerController.get().getApplication().getContext().removeTransactionListener(tl);
    }

    @SuppressWarnings("serial")
    private final class WindowTransactionListener implements TransactionListener {

        @Override
        public void transactionStart(Application application, Object transactionData) {
            if (application instanceof ConsumerApplication) {
                ConsumerController ctr = ((ConsumerApplication) application).getController();
                controller.set(ctr);
            } else {
                // This should not happen
                controller.set(null);
            }
        }

        @Override
        public void transactionEnd(Application application, Object transactionData) {
            if (application instanceof ConsumerApplication) {
                // do stuff here
                controller.remove();
            }
        }
    }

    public void updateContent(int msg) {
        application.getMainWindow().updateContent(msg);
    }

}
